package com.isu.gaswellwatch.modbus.data.listener;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.entity.RemindRecord;
import com.isu.gaswellwatch.entity.RemindSetting;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import com.isu.gaswellwatch.service.DeviceOptLogService;
import com.isu.gaswellwatch.service.DeviceService;
import com.isu.gaswellwatch.service.RemindService;
import com.isu.gaswellwatch.vo.DeviceVO;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.BatchMessageListener;
import org.springframework.amqp.core.Message;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
@RequiredArgsConstructor
public class BusinessMessageHandlerListener implements BatchMessageListener {

    @Resource
    private DeviceService deviceService;
    @Resource
    private DeviceOptLogService deviceOptLogService;
    @Resource
    private RemindService remindService;
    @Resource
    private SnowflakeConfig snowflakeConfig;


    @Override
    public void onMessage(Message message) {
        this.onMessageBatch(List.of(message));
    }

    @Override
    public void onMessageBatch(List<Message> messages) {

        //读取全部提醒字段的配置
        List<RemindSetting> settingList = this.remindService.getAllRemindSetting();
        List<RemindRecord> recordList = new ArrayList<>();

        //将settingList转换为map，key为deviceId，value为List<RemindSetting>
        Map<Long, List<RemindSetting>> deviceRemindSettingMap = new HashMap<>();
        for (RemindSetting setting : settingList) {
            if (deviceRemindSettingMap.containsKey(setting.getDeviceId())) {
                deviceRemindSettingMap.get(setting.getDeviceId()).add(setting);
            } else {
                deviceRemindSettingMap.put(setting.getDeviceId(), List.of(setting));
            }
        }


        Map<Long, Integer> deviceWellStatusMap = new HashMap<>();

        for (Message message : messages) {
            String messageString = new String(message.getBody());
            JSONObject messageObject = JSON.parseObject(messageString);
            if (messageObject.containsKey("old")) {
                JSONObject oldObject = messageObject.getJSONObject("old");
                JSONObject newObject = messageObject.getJSONObject("new");
                String deviceId = oldObject.getString("deviceId");
                if (StringUtils.isEmpty(deviceId)) {
                    continue;
                }
                //根据设备ID获取设备的产品类型
                DeviceVO device = this.deviceService.getDevice(Long.valueOf(deviceId));
                if(null == device){
                    continue;
                }
                //根据对应的产品类型，用对应的气井开关字段获取开关状态(wellStatus,solenoidValveStatus,firstSolenoidStatus)
                String wellStatusKey = getWellStatusKey(device);
                //比对新旧数据，看开关状态是否一致
                if (oldObject.getString(wellStatusKey) != null && (!oldObject.getString(wellStatusKey).equals(newObject.getString(wellStatusKey)))) {
                    deviceWellStatusMap.put(Long.valueOf(deviceId), newObject.getInteger(wellStatusKey));
                }

                //根据ID找到对应的提醒字段配置
                if (deviceRemindSettingMap.containsKey(Long.valueOf(deviceId))) {
                    List<RemindSetting> remindSettings = deviceRemindSettingMap.get(Long.valueOf(deviceId));
                    for (RemindSetting remindSetting : remindSettings) {
                        //遍历该id下的所有字段的配置，检查新消息中字段的值是否符合提醒条件
                        //暂时只处理number类型的字段
                        if ("number".equals(remindSetting.getFieldType())) {
                            BigDecimal newValue = newObject.getBigDecimal(remindSetting.getField());
                            if (newValue != null) {
                                if (!StringUtils.isEmpty(remindSetting.getMaxValue())) {
                                    if (newValue.compareTo(new BigDecimal(remindSetting.getMaxValue())) > 0) {
                                        //如果大于最大值，则保存提醒记录
                                        this.addRemindRecordToList(remindSetting, deviceId, device, newValue, recordList);
                                    } else if (!StringUtils.isEmpty(remindSetting.getMinValue())) {
                                        if (newValue.compareTo(new BigDecimal(remindSetting.getMinValue())) < 0) {
                                            //如果小于最小值，则保存提醒记录
                                            this.addRemindRecordToList(remindSetting, deviceId, device, newValue, recordList);
                                        }
                                    }
                                }
                            }
                        }


                    }
                }


            }

        }

        //若不一致则记录设备操作日志
        if (!deviceWellStatusMap.isEmpty()) {
            for (Map.Entry<Long, Integer> entry : deviceWellStatusMap.entrySet()) {
                this.deviceOptLogService.saveGasWellOptLog(entry.getValue(), entry.getKey(), "device");
            }
        }

        //如果提醒数据不为空，批量保存提醒记录
        if (!recordList.isEmpty()) {
            this.remindService.saveBatch(recordList);
        }
    }

    private void addRemindRecordToList(RemindSetting remindSetting, String deviceId, DeviceVO device, BigDecimal newValue, List<RemindRecord> recordList) {
        RemindRecord remindRecord = new RemindRecord();
        remindRecord.setId(this.snowflakeConfig.snowflakeId());
        remindRecord.setDeviceId(Long.valueOf(deviceId));
        remindRecord.setGasWell(device.getGasWell().getId());
        remindRecord.setContent("【" + remindSetting.getFieldName() + "】的值达到" + newValue + "，请及时处理");
        recordList.add(remindRecord);
    }

    private static String getWellStatusKey(DeviceVO device) {
        String wellStatusKey = "";
        if (PersistenceHandler.KNPCV1_MODBUS_TYPE.equals(device.getProduct().getCode())) {
            wellStatusKey = "wellStatus";
        } else if (PersistenceHandler.ETC_MODBUS_TYPE.equals(device.getProduct().getCode())) {
            wellStatusKey = "solenoidValveStatus";
        } else if (PersistenceHandler.SCSS_MODBUS_TYPE.equals(device.getProduct().getCode())) {
            wellStatusKey = "firstSolenoidStatus";
        }
        return wellStatusKey;
    }
}
