package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import jakarta.validation.constraints.NotBlank;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * 逻辑定值-柱塞模式定值数据寄存器地址
 *
 * @author <a href="mailto:scwsl@foxmail.com>王仕龙</a>
 * 2024/11/28 0:45
 */
@Getter
@Setter
@SuperBuilder
public class PlungerLogicMode extends Command implements Timing {

    @Serial
    private static final long serialVersionUID = -3715097823142330692L;

    /**
     * 柱塞上升时间
     */
    @NotBlank(message = "柱塞上升时间为空；格式：HH:mm:ss")
    private String plungerRiseDuration;
    /**
     * 续流时间
     */
    @NotBlank(message = "续流时间不能为空；格式：HH:mm:ss")
    private String continuosFlowDuration;
    /**
     * 关井时间
     */
    @NotBlank(message = "关井时间不能为空；格式：HH:mm:ss")
    private String wellCloseTimeDuration;
    /**
     * 未到达关井时间
     */
    @NotBlank(message = "未到达关井时间不能为空；格式：HH:mm:ss")
    private String wellCloseTimeNotReachedDuration;
    /**
     * 未到达次数
     */
    private int wellCloseNotReachedCountValue;
    /**
     * 柱塞延迟时间
     */
    private int plungerDelayDurationRepeat;
    /**
     * 目标时间
     */
    private int targetTimeTimestamp;
    /**
     * 目标时间范围
     */
    private int targetTimeRangeValue;
    /**
     * 续流增加时间
     */
    @NotBlank(message = "续流增加时间不能为空；格式：HH:mm:ss")
    private String continuosFlowIncreaseDuration;
    /**
     * 续流减少时间
     */
    @NotBlank(message = "最大开井时间不能为空；格式：HH:mm:ss")
    private String continuosFlowDecreaseDuration;
    /**
     * 关井增加时间
     */
    @NotBlank(message = "关井增加时间不能为空；格式：HH:mm:ss")
    private String wellCloseIncreaseDuration;
    /**
     * 关井减少时间
     */
    @NotBlank(message = "关井减少时间不能为空；格式：HH:mm:ss")
    private String wellCloseDecreaseDuration;
    /**
     * 最小关井时间
     */
    @NotBlank(message = "最小关井时间不能为空；格式：HH:mm:ss")
    private String minWellCloseTimeDuration2;
    /**
     * 最大关井时间
     */
    @NotBlank(message = "最大关井时间不能为空；格式：HH:mm:ss")
    private String maxWellCloseTimeDuration2;
    /**
     * 最小续流时间
     */
    @NotBlank(message = "最小续流时间不能为空；格式：HH:mm:ss")
    private String minContinuosFlowTimeDuration;
    /**
     * 最大续流时间
     */
    @NotBlank(message = "最大续流时间不能为空；格式：HH:mm:ss")
    private String maxContinuosFlowTimeDuration;
    /**
     * 最小开井时间
     */
    @NotBlank(message = "最小开井时间不能为空；格式：HH:mm:ss")
    private String minWellOpenTimeDuration2;
    /**
     * 最大开井时间
     */
    @NotBlank(message = "最大开井时间不能为空；格式：HH:mm:ss")
    private String maxWellOpenTimeDuration2;
    /**
     * 开井套压
     */
    private int wellOpenPressureValueAtOpen;
    /**
     * 开井油压
     */
    private int wellOpenOilPressureValueAtOpen;
    /**
     * 开井载荷因子预设值
     */
    private int wellOpenLoadFactorPresetsAtOpen;
    /**
     * 关井套压
     */
    private int wellClosePressureValueAtClose;
    /**
     * 关井油压
     */
    private int wellCloseOilPressureValueAtClose;
    /**
     * 关井流量
     */
    private int wellCloseFlowValueAtClose;

    public PlungerLogicMode() {
        this.setCode("SCSS.PLUNGER_LOGIC_MODE");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.plungerRiseDuration)
                && StringUtils.isNotBlank(this.continuosFlowDuration)
                && StringUtils.isNotBlank(this.wellCloseTimeDuration)
                && StringUtils.isNotBlank(this.wellCloseTimeNotReachedDuration)
                && StringUtils.isNotBlank(this.continuosFlowIncreaseDuration)
                && StringUtils.isNotBlank(this.continuosFlowDecreaseDuration)
                && StringUtils.isNotBlank(this.wellCloseIncreaseDuration)
                && StringUtils.isNotBlank(this.wellCloseDecreaseDuration)
                && StringUtils.isNotBlank(this.minWellCloseTimeDuration2)
                && StringUtils.isNotBlank(this.maxWellCloseTimeDuration2)
                && StringUtils.isNotBlank(this.minContinuosFlowTimeDuration)
                && StringUtils.isNotBlank(this.maxContinuosFlowTimeDuration)
                && StringUtils.isNotBlank(this.minWellOpenTimeDuration2)
                && StringUtils.isNotBlank(this.maxWellOpenTimeDuration2)
                && StringUtils.isNotBlank(this.minContinuosFlowTimeDuration);
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(130);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     00C8    0030 60
        command.append("011000C8003060");
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.plungerRiseDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.continuosFlowDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellCloseTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellCloseTimeNotReachedDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseNotReachedCountValue), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.plungerDelayDurationRepeat), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.targetTimeTimestamp), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.targetTimeRangeValue), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.continuosFlowIncreaseDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.continuosFlowDecreaseDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellCloseIncreaseDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.wellCloseDecreaseDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.minWellCloseTimeDuration2)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.maxWellCloseTimeDuration2)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.minContinuosFlowTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.maxContinuosFlowTimeDuration)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.minWellOpenTimeDuration2)), 8, "0"));
        command.append(StringUtils.leftPad(Long.toHexString(this.toSeconds(this.maxWellOpenTimeDuration2)), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenPressureValueAtOpen), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenOilPressureValueAtOpen), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellOpenLoadFactorPresetsAtOpen), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellClosePressureValueAtClose), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseOilPressureValueAtClose), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.wellCloseFlowValueAtClose), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setPlungerRiseDuration(String plungerRiseDuration) {
        this.plungerRiseDuration = this.timingValidate(plungerRiseDuration, "plungerRiseDuration");
    }

    public void setContinuosFlowDuration(String continuosFlowDuration) {
        this.continuosFlowDuration = this.timingValidate(continuosFlowDuration, "continuosFlowDuration");
    }

    public void setWellCloseTimeDuration(String wellCloseTimeDuration) {
        this.wellCloseTimeDuration = this.timingValidate(wellCloseTimeDuration, "wellCloseTimeDuration");
    }

    public void setWellCloseTimeNotReachedDuration(String wellCloseTimeNotReachedDuration) {
        this.wellCloseTimeNotReachedDuration = this.timingValidate(wellCloseTimeNotReachedDuration, "wellCloseTimeNotReachedDuration");
    }

    public void setContinuosFlowIncreaseDuration(String continuosFlowIncreaseDuration) {
        this.continuosFlowIncreaseDuration = this.timingValidate(continuosFlowIncreaseDuration, "continuosFlowIncreaseDuration");
    }

    public void setContinuosFlowDecreaseDuration(String continuosFlowDecreaseDuration) {
        this.continuosFlowDecreaseDuration = this.timingValidate(continuosFlowDecreaseDuration, "continuosFlowDecreaseDuration");
    }

    public void setWellCloseIncreaseDuration(String wellCloseIncreaseDuration) {
        this.wellCloseIncreaseDuration = this.timingValidate(wellCloseIncreaseDuration, "wellCloseIncreaseDuration");
    }

    public void setWellCloseDecreaseDuration(String wellCloseDecreaseDuration) {
        this.wellCloseDecreaseDuration = this.timingValidate(wellCloseDecreaseDuration, "wellCloseDecreaseDuration");
    }

    public void setMinWellCloseTimeDuration2(String minWellCloseTimeDuration2) {
        this.minWellCloseTimeDuration2 = this.timingValidate(minWellCloseTimeDuration2, "minWellCloseTimeDuration2");
    }

    public void setMaxWellCloseTimeDuration2(String maxWellCloseTimeDuration2) {
        this.maxWellCloseTimeDuration2 = this.timingValidate(maxWellCloseTimeDuration2, "maxWellCloseTimeDuration2");
    }

    public void setMinContinuosFlowTimeDuration(String minContinuosFlowTimeDuration) {
        this.minContinuosFlowTimeDuration = this.timingValidate(minContinuosFlowTimeDuration, "minContinuosFlowTimeDuration");
    }

    public void setMaxContinuosFlowTimeDuration(String maxContinuosFlowTimeDuration) {
        this.maxContinuosFlowTimeDuration = this.timingValidate(maxContinuosFlowTimeDuration, "maxContinuosFlowTimeDuration");
    }

    public void setMinWellOpenTimeDuration2(String minWellOpenTimeDuration2) {
        this.minWellOpenTimeDuration2 = this.timingValidate(minWellOpenTimeDuration2, "minWellOpenTimeDuration2");
    }

    public void setMaxWellOpenTimeDuration2(String maxWellOpenTimeDuration2) {
        this.maxWellOpenTimeDuration2 = this.timingValidate(maxWellOpenTimeDuration2, "maxWellOpenTimeDuration2");
    }

}
